/*
  dmp/device.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "component.h"

#include <cstdint>
#include <vector>

namespace ACN::DMP {

/**
 * @brief The ACN::DMP::Device class
 */
class Device
  : public virtual DMP::Component
{
public:
  explicit Device(UUID::uuid = UUID::uuid(), std::string fctn = "OpenLCP DMP Device");
  virtual ~Device();

protected:
  virtual void rxDmpGetProperty(PDU::Message<DMP::Pdu>) override;
  virtual uint32_t getProperty(const Range, failure_reason * = nullptr);

  virtual void rxDmpSetProperty(PDU::Message<DMP::Pdu>) override;
  virtual void setProperty(const address_data_pair, failure_reason * = nullptr);

private:
  /// \cite DMP 5.1.1 Property addresses
  /// Within a component, properties are addressed in a one dimensional
  /// property array.
  std::vector<uint32_t> properties_;
};

} // namespace ACN::DMP
